@@ -19,12 +19,12 @@ import (
 	"fmt"
 	"log/slog"
 	"math"
-	"os"
 	"strconv"
 	"time"
 	"unicode/utf8"
 
-	_ "github.com/lib/pq"
+	"github.com/lib/pq"
+
 	"github.com/prometheus/client_golang/prometheus"
 )
 
@@ -44,16 +44,20 @@ var (
 			"paused":              {GAUGE, "paused", 1, "1 if this database is currently paused, else 0"},
 			"disabled":            {GAUGE, "disabled", 1, "1 if this database is currently disabled, else 0"},
 		},
-		"stats": {
-			"database":          {LABEL, "N/A", 1, "N/A"},
-			"total_query_count": {COUNTER, "queries_pooled_total", 1, "Total number of SQL queries pooled"},
-			"total_query_time":  {COUNTER, "queries_duration_seconds_total", 1e-6, "Total number of seconds spent by pgbouncer when actively connected to PostgreSQL, executing queries"},
-			"total_received":    {COUNTER, "received_bytes_total", 1, "Total volume in bytes of network traffic received by pgbouncer, shown as bytes"},
-			"total_requests":    {COUNTER, "queries_total", 1, "Total number of SQL requests pooled by pgbouncer, shown as requests"},
-			"total_sent":        {COUNTER, "sent_bytes_total", 1, "Total volume in bytes of network traffic sent by pgbouncer, shown as bytes"},
-			"total_wait_time":   {COUNTER, "client_wait_seconds_total", 1e-6, "Time spent by clients waiting for a server in seconds"},
-			"total_xact_count":  {COUNTER, "sql_transactions_pooled_total", 1, "Total number of SQL transactions pooled"},
-			"total_xact_time":   {COUNTER, "server_in_transaction_seconds_total", 1e-6, "Total number of seconds spent by pgbouncer when connected to PostgreSQL in a transaction, either idle in transaction or executing queries"},
+		"stats_totals": {
+			"database":                      {LABEL, "N/A", 1, "N/A"},
+			"query_count":                   {COUNTER, "queries_pooled_total", 1, "Total number of SQL queries pooled"},
+			"query_time":                    {COUNTER, "queries_duration_seconds_total", 1e-6, "Total number of seconds spent by pgbouncer when actively connected to PostgreSQL, executing queries"},
+			"bytes_received":                {COUNTER, "received_bytes_total", 1, "Total volume in bytes of network traffic received by pgbouncer, shown as bytes"},
+			"requests":                      {COUNTER, "queries_total", 1, "Total number of SQL requests pooled by pgbouncer, shown as requests"},
+			"bytes_sent":                    {COUNTER, "sent_bytes_total", 1, "Total volume in bytes of network traffic sent by pgbouncer, shown as bytes"},
+			"wait_time":                     {COUNTER, "client_wait_seconds_total", 1e-6, "Time spent by clients waiting for a server in seconds"},
+			"xact_count":                    {COUNTER, "sql_transactions_pooled_total", 1, "Total number of SQL transactions pooled"},
+			"xact_time":                     {COUNTER, "server_in_transaction_seconds_total", 1e-6, "Total number of seconds spent by pgbouncer when connected to PostgreSQL in a transaction, either idle in transaction or executing queries"},
+			"client_parse_count":            {COUNTER, "client_parses_total", 1, "Total number of prepared statement Parse messages received from clients"},
+			"server_parse_count":            {COUNTER, "server_parses_total", 1, "Total number of prepared statement Parse messages sent by pgbouncer to PostgreSQL"},
+			"bind_count":                    {COUNTER, "binds_total", 1, "Total number of prepared statements readied for execution with a Bind message"},
+			"total_server_assignment_count": {COUNTER, "server_assignments_total", 1, "Total number of client connections which have been served since process start"},
 		},
 		"pools": {
 			"database":              {LABEL, "N/A", 1, "N/A"},
@@ -134,17 +138,15 @@ var (
 )
 
 func NewExporter(connectionString string, namespace string, logger *slog.Logger) *Exporter {
-
-	db, err := getDB(connectionString)
-
+	conn, err := pq.NewConnector(connectionString)
 	if err != nil {
-		logger.Error("error setting up DB connection", "err", err.Error())
-		os.Exit(1)
+		logger.Error("failed to create connector", "error", err)
+		return nil
 	}
 
 	return &Exporter{
+		conn:      conn,
 		metricMap: makeDescMap(metricMaps, namespace, logger),
-		db:        db,
 		logger:    logger,
 	}
 }
@@ -327,15 +329,15 @@ func queryNamespaceMapping(ch chan<- prometheus.Metric, db *sql.DB, namespace st
 	}
 	if err := rows.Err(); err != nil {
 		logger.Error("Failed scaning all rows", "err", err.Error())
-		nonfatalErrors = append(nonfatalErrors, fmt.Errorf("Failed to consume all rows due to: %w", err))
+		nonfatalErrors = append(nonfatalErrors, fmt.Errorf("failed to consume all rows due to: %w", err))
 	}
 	return nonfatalErrors, nil
 }
 
-func getDB(conn string) (*sql.DB, error) {
-	db, err := sql.Open("postgres", conn)
-	if err != nil {
-		return nil, err
+func getDB(conn *pq.Connector) (*sql.DB, error) {
+	db := sql.OpenDB(conn)
+	if db == nil {
+		return nil, errors.New("error opening DB")
 	}
 	rows, err := db.Query("SHOW STATS")
 	if err != nil {
@@ -469,36 +471,47 @@ func (e *Exporter) Describe(ch chan<- *prometheus.Desc) {
 
 // Collect implements prometheus.Collector.
 func (e *Exporter) Collect(ch chan<- prometheus.Metric) {
-	e.logger.Info("Starting scrape")
+	e.logger.Debug("Starting scrape")
 
 	var up = 1.0
 
-	err := queryVersion(ch, e.db)
+	defer func() {
+		ch <- prometheus.MustNewConstMetric(scrapeSuccessDesc, prometheus.GaugeValue, up)
+	}()
+
+	db, err := getDB(e.conn)
 	if err != nil {
-		e.logger.Error("error getting version", "err", err.Error())
+		e.logger.Warn("error setting up DB connection", "err", err.Error())
+		up = 0
+		return
+	}
+	defer db.Close()
+
+	err = queryVersion(ch, db)
+	if err != nil {
+		e.logger.Warn("error getting version", "err", err.Error())
 		up = 0
 	}
 
-	if err = queryShowLists(ch, e.db, e.logger); err != nil {
-		e.logger.Error("error getting SHOW LISTS", "err", err.Error())
+	if err = queryShowLists(ch, db, e.logger); err != nil {
+		e.logger.Warn("error getting SHOW LISTS", "err", err.Error())
 		up = 0
 	}
 
-	if err = queryShowConfig(ch, e.db, e.logger); err != nil {
-		e.logger.Error("error getting SHOW CONFIG", "err", err.Error())
+	if err = queryShowConfig(ch, db, e.logger); err != nil {
+		e.logger.Warn("error getting SHOW CONFIG", "err", err.Error())
 		up = 0
 	}
 
-	errMap := queryNamespaceMappings(ch, e.db, e.metricMap, e.logger)
+	errMap := queryNamespaceMappings(ch, db, e.metricMap, e.logger)
 	if len(errMap) > 0 {
-		e.logger.Error("error querying namespace mappings", "err", errMap)
+		e.logger.Warn("error querying namespace mappings", "err", errMap)
 		up = 0
 	}
 
 	if len(errMap) == len(e.metricMap) {
 		up = 0
 	}
-	ch <- prometheus.MustNewConstMetric(scrapeSuccessDesc, prometheus.GaugeValue, up)
 }
 
 // Turn the MetricMap column mapping into a prometheus descriptor mapping.
